@@ -4,12 +4,12 @@ set -e
 cd "$(dirname ${BASH_SOURCE[0]})"
 
 # Copy in files that would otherwise need to download.
-ln -f ci/*.* skyfield/documentation
+ln -f ci/*.* documentation/
 
 function cleanup {
-    cd skyfield/documentation
-    rm -f $(cd ../../ci; ls *.*)
+    cd documentation
+    rm -f $(cd ../ci && ls *.*)
 }
 trap cleanup EXIT
 
-make -C skyfield/documentation doctest
+make -C documentation doctest
